==> snips-sam-0.0.24.tgz: package/src/commands/setup/audio.ts <==
import chalk from 'chalk'
import * as yargs from 'yargs'
import { cli } from '../../cli'
import { AudioDevice } from '../../utils/audiodevice'
import { SSHService, SnipsServices } from '../../session/ssh'

exports.command = 'audio'
exports.desc = 'Quickly setup the mic & speaker on your device'

exports.handler = async (_: yargs.Argv) => {
    const ssh = new SSHService()
    await ssh.connect()
    .catch(e => {
        cli.stream.error(e)
        process.exit()
    })

    const isMakersKit = await cli.prompt.promptQuestions([
        {
            name: 'makersKit',
            type: 'confirm',
            message: `Is it a Snips Makers Kit?`,
        },
    ])
    .then(makersKit => {
        const response: boolean = makersKit['makersKit']
        return response
    })

    if (isMakersKit) {
        cli.stream.loading('Installing ReSpeaker 2 mic hat')
        await ssh.checkCommandExists('git')
        .then(isGitInstalled => {
            if (!isGitInstalled) return ssh.aptGetY(['git'], _ => {})
            return ''
        })
        .then(_ => ssh.installRespeakerMicHat(_ => {}))
        .then(_ => cli.stream.success())
        .then(_ => ssh.hasSnipsSkillsServerGroup())
        .then(hasSnipsSkillsGroup => {
            if (hasSnipsSkillsGroup) return ''
            return ssh.aptGetY([SnipsServices.template, SnipsServices.skillServer], _ => {})
        })
        .then(_ => ssh.run('sudo usermod -a -G spi,gpio,audio _snips-skills'))
        .catch(e => cli.stream.error(e.message))

        await ssh.reboot().catch(_ => {})
        let loadingText = 'Rebooting device.'
        cli.stream.loading(loadingText)
        await ssh.retryConnect(() => {
            loadingText += ' . '
            cli.stream.update(loadingText)
        })
        .then(isConnected => {
            if (isConnected) {
                cli.stream.done()
                cli.stream.success('Device has rebooted')
            } else {
                cli.stream.error('Failed to reconnect, try a hard reboot')
            }
        })
    }

    let captureDevice: AudioDevice | undefined
    try {
        const captureDevices = await ssh.listCaptureDevices()
        if (captureDevices.length === 1) {
            cli.stream.success(`Found only 1 capture device named: ${captureDevices[0].name}`)
            cli.stream.hint(`Using this device to capture sound`)
            captureDevice = captureDevices[0]
        } else {
            const response = await cli.prompt.promptList(
                'device',
                `Found ${captureDevices.length} interfaces to record sound.\n  Choose the one you want to use`,
                captureDevices.map(device => device.name),
            )
            captureDevice = captureDevices.find(device => device.name === response['device'])
        }
    } catch (e) {
        cli.stream.error('Error fetching audio capture device: ' + e.message)
        cli.stream.hint('If the microphone is not connected through USB, you might need to install it yourself. See the manufacturers installation guide')
    }
    if (captureDevice === undefined) {
        cli.stream.hint('Using default capture configuration card 1 subdevice 0')
        captureDevice = { name: 'Default', card: 1, subdevice: 0 }
    }

    let outputDevice: AudioDevice | undefined
    try {
        const outputDevices = await ssh.listOuputDevices()
        if (outputDevices.length === 1) {
            cli.stream.success(`Found only 1 output device named: ${outputDevices[0].name}`)
            cli.stream.hint(`Using this device to output sound`)
            outputDevice = outputDevices[0]
        } else {
            const response = await cli.prompt.promptList(
                'device',
                `Found ${outputDevices.length} interfaces to output sound.\n  Choose the one you want to use as your speaker`,
                outputDevices.map(device => device.name),
            )
            outputDevice = outputDevices.find(device => device.name === response['device'])
        }
    } catch (e) {
        cli.stream.error('Error fetching audio output device: ' + e.message)
    }
    if (outputDevice === undefined) {
        cli.stream.error(`An output device wasn't found`)
        return
    }

    try {
        await ssh.setupAsoundConf(outputDevice, captureDevice)
        cli.stream.success('Installed /etc/asound.conf')
    } catch (e) {
        cli.stream.error('Error installing the microphone asound.conf: ' + e.message)
        ssh.disconnect()
        process.exit()
    }

    cli.stream.println('Setting to volume to 90% if possible')
    await ssh.setVolumeLevel(outputDevice.card, 90).catch(_ => {})

    cli.stream.loading('Restarting snips-audio-server service')
    await ssh.relaunchService(SnipsServices.audioServer).catch(_ => {})
    cli.stream.done()

    cli.stream.hint(chalk`You can test your microphone & speaker with {blue sam test microphone / speaker}`)
    ssh.disconnect()
}

==> snips-sam-0.0.26.tgz: package/src/commands/setup/audio.ts <==
import chalk from 'chalk'
import * as yargs from 'yargs'
import { cli } from '../../cli'
import { AudioDevice } from '../../utils/audiodevice'
import { SSHService, SnipsServices } from '../../session/ssh'

exports.command = 'audio'
exports.desc = 'Quickly setup the mic & speaker on your device'

exports.handler = async (_: yargs.Argv) => {
    const ssh = new SSHService()
    await ssh.connect()
    .catch(e => {
        cli.stream.error(e)
        process.exit()
    })

    const isMakersKit = await cli.prompt.promptQuestions([
        {
            name: 'makersKit',
            type: 'confirm',
            message: `Is it a Snips Makers Kit?`,
        },
    ])
    .then(makersKit => {
        const response: boolean = makersKit['makersKit']
        return response
    })

    if (isMakersKit) {
        cli.stream.loading('Installing ReSpeaker 2 mic hat')

        try {
            const isGitInstalled = await ssh.checkCommandExists('git')

            if (!isGitInstalled) {
                await ssh.aptGetY(['git'], _ => {})
            }

            await ssh.installRespeakerMicHat(_ => {})
            cli.stream.success()
            const hasSnipsSkillsGroup = await ssh.dpkgCheckIfPackageIsInstalled(SnipsServices.template)

            if (!hasSnipsSkillsGroup) {
                await ssh.aptGetY([SnipsServices.template, SnipsServices.skillServer], _ => {})
            }

            await ssh.addCurrentUserToSnipsSkillsAdminGroup()
            await ssh.run('sudo usermod -a -G spi,gpio,audio _snips-skills')
            cli.stream.loading('Installing ReSpeaker Snips skill. This could take a while.')
            const isVirtualEnvInstalled = await ssh.checkCommandExists('virtualenv')

            if (!isVirtualEnvInstalled) {
                await ssh.installPythonAndPip(_ => {})
                cli.stream.success()
            }

            await ssh.gitClone('https://github.com/snipsco/snips-skill-respeaker.git', undefined, _ => {})
            await ssh.run('sudo raspi-config nonint do_spi 1')
            await ssh.stopService(SnipsServices.skillServer)
            await ssh.runSkillSetupSH()

            cli.stream.success()
        } catch (e) {
            cli.stream.error(e.message)
        }

        await ssh.reboot().catch(_ => {})

        let loadingText = 'Rebooting device.'
        cli.stream.loading(loadingText)
        const isConnected = await ssh.retryConnect(
            retries => {
                loadingText += ' . '
                cli.stream.update(loadingText)
                // After 30s, the device may be stuck so we ask the user to reboot manually.
                if (retries === 6) {
                    cli.stream.println()
                    cli.stream.hint('This seems to be taking a long time, please unplug and power up the raspberry manually.')
                    loadingText = ''
                }
            },
            30)

        if (isConnected) {
            cli.stream.done()
            cli.stream.success('Device has rebooted')
        } else {
            cli.stream.error('Failed to reconnect, try a hard reboot')
        }
    }

    let captureDevice: AudioDevice | undefined
    try {
        const captureDevices = await ssh.listCaptureDevices()
        if (captureDevices.length === 1) {
            cli.stream.success(`Found only 1 capture device named: ${captureDevices[0].name}`)
            cli.stream.hint(`Using this device to capture sound`)
            captureDevice = captureDevices[0]
        } else {
            const response = await cli.prompt.promptList(
                'device',
                `Found ${captureDevices.length} interfaces to record sound.\n  Choose the one you want to use`,
                captureDevices.map(device => device.name),
            )
            captureDevice = captureDevices.find(device => device.name === response['device'])
        }
    } catch (e) {
        cli.stream.error('Error fetching audio capture device: ' + e.message)
        cli.stream.hint('If the microphone is not connected through USB, you might need to install it yourself. See the manufacturers installation guide')
    }
    if (captureDevice === undefined) {
        cli.stream.hint('Using default capture configuration card 1 subdevice 0')
        captureDevice = { name: 'Default', card: 1, subdevice: 0 }
    }

    let outputDevice: AudioDevice | undefined
    try {
        const outputDevices = await ssh.listOuputDevices()
        if (outputDevices.length === 1) {
            cli.stream.success(`Found only 1 output device named: ${outputDevices[0].name}`)
            cli.stream.hint(`Using this device to output sound`)
            outputDevice = outputDevices[0]
        } else {
            const response = await cli.prompt.promptList(
                'device',
                `Found ${outputDevices.length} interfaces to output sound.\n  Choose the one you want to use as your speaker`,
                outputDevices.map(device => device.name),
            )
            outputDevice = outputDevices.find(device => device.name === response['device'])
        }
    } catch (e) {
        cli.stream.error('Error fetching audio output device: ' + e.message)
    }
    if (outputDevice === undefined) {
        cli.stream.error(`An output device wasn't found`)
        return
    }

    try {
        await ssh.setupAsoundConf(outputDevice, captureDevice)
        cli.stream.success('Installed /etc/asound.conf')
    } catch (e) {
        cli.stream.error('Error installing the microphone asound.conf: ' + e.message)
        ssh.disconnect()
        process.exit()
    }

    cli.stream.println('Setting to volume to 90% if possible')
    await ssh.setVolumeLevel(outputDevice.card, 90).catch(_ => {})

    cli.stream.loading('Restarting snips-audio-server service')
    await ssh.relaunchService(SnipsServices.audioServer).catch(_ => {})
    cli.stream.done()

    cli.stream.hint(chalk`You can test your microphone & speaker with {blue sam test microphone / speaker}`)
    ssh.disconnect()
}
